@@ -1,7 +1,9 @@
+import os
 import shlex
 from pathlib import Path
-from typing import List
+from typing import List, Optional, TypedDict
 
+import pkg_resources
 import yaml
 
 from . import file_contents, helpers, steps
@@ -13,7 +15,7 @@ class Runtime:
         self.config = config
         self.operation_id = operation_id
         self.entry_point = kwargs.get('entry_point', 'main')
-        self.runtime = kwargs.get('runtime', 'python311')
+        self.runtime = kwargs.get('runtime', 'python312')
         self.memory = kwargs.get('memory', '256MB')
         self.dependencies = kwargs.get('dependencies', [])
         self.trigger = kwargs.get('trigger', 'http')
@@ -21,10 +23,14 @@ class Runtime:
             self.trigger = '--trigger-http'
         else:
             self.trigger = f"--trigger-topic='{self.trigger}'"
+        self.vpc_connector = kwargs.get('vpc_connector', None)
 
     @property
     def command(self) -> str:
-        return f'gcloud functions deploy --project="{self.config.project}" {self.operation_id} {self.trigger} --env-vars-file env.yaml --runtime {self.runtime} --entry-point="{self.entry_point}" --memory={self.memory} --region="{self.config.region}"'
+        vpc_connector = ''
+        if self.vpc_connector is not None:
+            vpc_connector = f'--vpc-connector="{self.vpc_connector}"'
+        return f'gcloud functions deploy --project="{self.config.project}" {self.operation_id} {self.trigger} --env-vars-file env.yaml --runtime {self.runtime} --entry-point="{self.entry_point}" --memory={self.memory} --region="{self.config.region}" --no-allow-unauthenticated {vpc_connector}'.strip()
 
 
 class Endpoint:
@@ -79,20 +85,48 @@ def endpoint_list(config: Config) -> List[Endpoint]:
     return [item for sublist in endpoints for item in sublist]
 
 
-def endpoint_create(base_dir: str, name: str):
+def endpoint_append_api(config: Config, name: str):
+    api_config = endpoint_load(config)
+    api_config['paths'][f'/{name}'] = yaml.safe_load(
+        file_contents.api_endpoint(
+            name,
+            project=config.project,
+            region=config.region,
+        )
+    )
+    with open(f"{config.base_dir}/api.yaml", "w") as file:
+        file.write(yaml.dump(
+            api_config,
+            explicit_start=True,
+            sort_keys=False,
+            indent=2,
+            width=1000
+        ))
+
+
+def endpoint_create(config: Config, name: str):
     endpoint_path = 'endpoints'
-    if not Path(base_dir, 'src', 'services', f'{name}.py').exists():
-        with open(f"{base_dir}/{endpoint_path}/src/services/{name}.py", "w") as file:
+    if Path(config.base_dir, endpoint_path, 'src', 'services', '__init__.py').exists():
+        with open(f"{config.base_dir}/{endpoint_path}/src/services/__init__.py", "a") as file:
+            file.write(f"\nfrom . import {name}\n")
+    if not Path(config.base_dir, 'src', 'services', f'{name}.py').exists():
+        with open(f"{config.base_dir}/{endpoint_path}/src/services/{name}.py", "w") as file:
             file.write(file_contents.endpoint_lib(name))
-    if not Path(base_dir, endpoint_path).exists():
-        Path(base_dir, endpoint_path).mkdir()
-    if not Path(base_dir, f'{endpoint_path}/{name}.py').exists():
-        with open(f"{base_dir}/{endpoint_path}/{name}.py", "w") as file:
+    if not Path(config.base_dir, endpoint_path).exists():
+        Path(config.base_dir, endpoint_path).mkdir()
+    if not Path(config.base_dir, f'{endpoint_path}/{name}.py').exists():
+        with open(f"{config.base_dir}/{endpoint_path}/{name}.py", "w") as file:
             file.write(file_contents.endpoint_main(name))
-    if not Path(base_dir, f'{endpoint_path}/{name}.yaml').exists():
-        with open(f"{base_dir}/{endpoint_path}/{name}.yaml", "w") as file:
+    if not Path(config.base_dir, f'{endpoint_path}/{name}_test.py').exists():
+        with open(f"{config.base_dir}/{endpoint_path}/{name}_test.py", "w") as file:
+            file.write(file_contents.endpoint_test(name))
+    if not Path(config.base_dir, f'{endpoint_path}/{name}.yaml').exists():
+        with open(f"{config.base_dir}/{endpoint_path}/{name}.yaml", "w") as file:
             file.write(file_contents.endpoint_config(name))
 
+    if name not in ['docs', 'health']:
+        endpoint_append_api(config, name)
+
 
 def init_file_structure(config: Config):
     base_dir = config.base_dir
@@ -111,14 +145,14 @@ def init_file_structure(config: Config):
             file.write("""from . import docs
 from . import health
 """)
-        endpoint_create(base_dir, 'health')
-        endpoint_create(base_dir, 'docs')
+        endpoint_create(config, 'health')
+        endpoint_create(config, 'docs')
 
     if not Path(base_dir, 'api.yaml').exists():
         with open(f"{base_dir}/api.yaml", "w") as file:
             file.write(file_contents.api(
-                title=config.endpoint_api_title,
-                description=config.endpoint_api_description,
+                api_title=config.endpoint_api_title,
+                api_description=config.endpoint_api_description,
                 project=config.project,
                 region=config.region,
             ))
@@ -130,10 +164,52 @@ from . import health
             file.write(file_contents.requirements())
 
 
+class Dependency(TypedDict):
+    name: str
+    is_installed: bool
+    version: Optional[str]
+
+
+def check_dependencies_exists(name: List[str]) -> List[Dependency]:
+    def check(package_name: str):
+        try:
+            dist = pkg_resources.get_distribution(package_name)
+            return {
+                'name': package_name,
+                'is_installed': True,
+                'version': dist.version,
+            }
+        except pkg_resources.DistributionNotFound:
+            return {
+                'name': package_name,
+                'is_installed': False,
+                'version': None,
+            }
+
+    return [check(x) for x in name]
+
+
+def check_dependencies() -> bool:
+    dependency_data = check_dependencies_exists([
+        'functions-framework',
+        'pytest',
+    ])
+    dependencies_not_installed = [x for x in dependency_data if not x['is_installed']]
+    if len(dependencies_not_installed) > 0:
+        print(helpers.bcolors.FAIL + 'Missing python dependencies:' + helpers.bcolors.ENDC)
+        for dependency in dependencies_not_installed:
+            print(f"+ {dependency['name']}")
+        return False
+    return True
+
+
 def cli_main(args, config: Config) -> int:
     if not config.load():
         return 1
 
+    if not check_dependencies():
+        return 1
+
     if args.action == 'init':
         manager = steps.Manager(config, [
             steps.StepFunction(
@@ -170,7 +246,7 @@ def cli_main(args, config: Config) -> int:
                 + helpers.bcolors.ENDC
             )
             return 1
-        endpoint_create(config.base_dir, args.endpoint)
+        endpoint_create(config, args.endpoint)
         print(helpers.bcolors.OKGREEN + 'Endpoint created' + helpers.bcolors.ENDC)
     elif args.action == 'describe':
         endpoints = [x for x in endpoint_list(config) if x.operation_id == args.endpoint]
@@ -227,10 +303,12 @@ def cli_main(args, config: Config) -> int:
             steps.StepCommand(
                 shlex.split("secrethub inject -i env.temp.yaml -o env.yaml"),
                 work_dir=f'{config.base_dir}/build/endpoints/{endpoint.operation_id}',
+                is_async=False,
             ),
             steps.StepCommand(
                 shlex.split(runtime.command),
                 work_dir=f'{config.base_dir}/build/endpoints/{endpoint.operation_id}',
+                is_async=False,
             ),
         ], simulate=args.simulate)
         rc = manager.run()
@@ -259,3 +337,47 @@ def cli_main(args, config: Config) -> int:
         rc = manager.run()
         if rc != 0:
             return rc
+    elif args.action == 'test':
+        endpoints = [x for x in endpoint_list(config) if x.operation_id == args.endpoint]
+        if len(endpoints) == 0:
+            print(
+                helpers.bcolors.FAIL
+                + f'Endpoint {args.endpoint} not found'
+                + helpers.bcolors.ENDC
+            )
+            return 1
+        endpoint = endpoints[0]
+
+        path = f'{config.base_dir}/endpoints/{endpoint.operation_id}_test.py'
+        if not os.path.exists(f'{config.base_dir}/endpoints/{endpoint.operation_id}_test.py'):
+            print(
+                helpers.bcolors.FAIL
+                + f'Endpoint test {path} not exists'
+                + helpers.bcolors.ENDC
+            )
+            return 1
+
+        print('# Test')
+        interpreter = 'python'
+
+        venv_bin = 'venv/bin/python'
+        if os.path.exists(venv_bin):
+            interpreter = os.path.abspath('venv/bin/python')
+        else:
+            print(
+                helpers.bcolors.WARNING
+                + f'Venv {venv_bin} not found use default python interpreter'
+                + helpers.bcolors.ENDC
+            )
+
+        manager = steps.Manager(config, [
+            steps.StepCommand(
+                shlex.split(f"{interpreter} {endpoint.operation_id}_test.py"),
+                work_dir=f'{config.base_dir}/endpoints',
+                verbose=True,
+                is_async=False,
+            ),
+        ], simulate=args.simulate)
+        rc = manager.run()
+        if rc != 0:
+            return rc
