@@ -1,4 +1,48 @@
+def api_endpoint(name: str, **kwargs) -> str:
+  """
+    Create an API endpoint.
+
+    Parameters:
+    - name (str): Name of the endpoint.
+    - project (str): Project name.
+    - region (str): Region name.
+
+    Returns:
+    - str: The content of the endpoint in Swagger OpenAPI 2.0 format
+    """
+  project = kwargs.get('project')
+  region = kwargs.get('region')
+  content = f"""
+get:
+  tags:
+  - "Main"
+  summary: {name} Endpoint
+  operationId: {name}
+  consumes:
+  - "application/json"
+  x-google-backend:
+      address: https://{region}-{project}.cloudfunctions.net/{name}
+  security:
+  - api_key: []
+  responses:
+      "200":
+      description: A successful response
+    """
+  return content
+
 def api(**kwargs):
+    """
+    This function creates an Swagger API.
+
+    Parameters:
+    - api_title (str): Title of the API (default: 'My API')
+    - api_description (str): Description of the API (default: 'My API Description')
+    - project (str): Project name
+    - region (str): Region name
+
+    Returns:
+    - str: The content of the endpoint in Swagger OpenAPI 2.0 format.
+    """
     api_title = kwargs.get('api_title', 'My API')
     api_description = kwargs.get('api_description', 'My API Description')
     project = kwargs.get('project')
@@ -93,8 +137,7 @@ def main(request):
 
 """
 
-    return """
-import importlib
+    return """import importlib
 import functions_framework
 from flask import jsonify
 
@@ -111,6 +154,27 @@ def main(request):
 """.replace('#SERVICE#', name)
 
 
+def endpoint_test(name: str, **kwargs):
+    return """
+import pytest
+from shopcloud_serverless import endpoints_utils
+
+
+class Test(endpoints_utils.TestEndpointAPIIntegration):
+    FILE = __file__
+
+    def test_main(self):
+        response = self.client.get("/")
+        assert response.status_code == 200
+
+
+if __name__ == "__main__":
+    exit_code = pytest.main([__file__])
+    exit(exit_code)
+
+""".replace('#SERVICE#', name)
+
+
 def endpoint_config(name: str, **kwargs):
     return """trigger: http
 """
@@ -124,7 +188,7 @@ def endpoint_lib(name: str, **kwargs):
 <head>
 <link type="text/css" rel="stylesheet" href="https://cdn.jsdelivr.net/npm/swagger-ui-dist@4/swagger-ui.css">
 <link rel="shortcut icon" href="https://fastapi.tiangolo.com/img/favicon.png">
-<title>Sage-Gateway - Swagger UI</title>
+<title>Swagger UI</title>
 </head>
 <body>
 <div id="swagger-ui">
