@@ -1,45 +1,43 @@
 import shlex
 
-from . import exceptions, helpers, steps
+from . import helpers, steps
 from .configs import Config
 
 
 def cli_main(args, config: Config) -> int:
-    try:
-        config.load()
-    except exceptions.ConfigInvalidVersion:
-        print(
-            helpers.bcolors.FAIL
-            + 'Config file is not compatible with this version. Please run `init` again.'
-            + helpers.bcolors.ENDC
-        )
+    if not config.load():
         return 1
     if args.action == 'init':
         manager = steps.Manager(config, [
             steps.StepCommand(
                 shlex.split(
                     f'gcloud services enable apigateway.googleapis.com --project="{config.project}"'
-                )
+                ),
+                is_async=False,
             ),
             steps.StepCommand(
                 shlex.split(
                     f'gcloud services enable servicemanagement.googleapis.com --project="{config.project}"'
-                )
+                ),
+                is_async=False,
             ),
             steps.StepCommand(
                 shlex.split(
                     f'gcloud services enable servicecontrol.googleapis.com --project="{config.project}"'
-                )
+                ),
+                is_async=False,
             ),
             steps.StepCommand(
                 shlex.split(
                     f'gcloud services enable cloudbuild.googleapis.com --project="{config.project}"'
-                )
+                ),
+                is_async=False,
             ),
             steps.StepCommand(
                 shlex.split(
                     f'gcloud services enable cloudfunctions.googleapis.com --project="{config.project}"'
-                )
+                ),
+                is_async=False,
             ),
         ], simulate=args.simulate)
         rc = manager.run()
@@ -61,12 +59,14 @@ def cli_main(args, config: Config) -> int:
                         f'gcloud api-gateway api-configs create {api_config_id} --api={api_id} --openapi-spec=api.yaml --project="{config.project}" --backend-auth-service-account="{config.project}@appspot.gserviceaccount.com"'
                     ),
                     work_dir=f'{config.base_dir}',
+                    is_async=False,
                 ),
                 steps.StepCommand(
                     shlex.split(
                         f'gcloud api-gateway gateways create {api_id} --api={api_id} --api-config={api_config_id} --location={config.region} --project="{config.project}"'
                     ),
                     work_dir=f'{config.base_dir}',
+                    is_async=False,
                 )
             ]
         else:
@@ -78,12 +78,14 @@ def cli_main(args, config: Config) -> int:
                         f'gcloud api-gateway api-configs create {api_config_id} --api={api_id} --openapi-spec=api.yaml --project="{config.project}" --backend-auth-service-account="{config.project}@appspot.gserviceaccount.com"'
                     ),
                     work_dir=f'{config.base_dir}',
+                    is_async=False,
                 ),
                 steps.StepCommand(
                     shlex.split(
                         f'gcloud api-gateway gateways update {api_id} --api={api_id} --api-config={api_config_id} --location={config.region} --project="{config.project}"'
                     ),
                     work_dir=f'{config.base_dir}',
+                    is_async=False,
                 )
             ]
 
